<?php
/**
 * @package rgraph Joomla! Plugin
 * @version Version 1
 * @author  Joachim Schmidt - joachim.schmidt@jschmidt-systemberatung.de
 * @copyright (C) 2016 Joachim Schmidt. All rights reserved.
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @abstract:   json php-helper class for rgraph charts package
 *
 */
namespace rgraph\lib;
// no direct access
defined('_JEXEC') or die('Restricted access');
include_once 'rgraph_valid_options.php';

class plg_rgraph_options
{

    function __construct ($options = null)
    {
        $this->rgraph_options($options);
    }

    function rgraph_options ($options)
    {
        // common options
        if (isset($options))
        {
            $this->debuginfo = array();
            $this->debug = false;
            $this->set_optionsviaArray($options, false);
        }
        else
        {
            $this->debug = false;
            $this->debuginfo = array();
            $this->marginTop = 50;
            $this->marginBottom = 50;
            $this->marginLeft = 50;
            $this->marginRight = 50;
            $this->scaleDecimals = 0;
            $this->scalePoint = ".";
            $this->scaleThousand = ",";
            $this->colors = array(
                    "rgb(0,0,255)",
                    "#0f0",
                    "#00f",
                    "#ff0",
                    "#0ff",
                    "#0f0"
            );
        }
    }

    public function get_option ($option)
    {
        if (isset($this->$option))
            return $this->$option;
    }

    public function set_debuginfo ($info)
    {
        array_push($this->debuginfo, $info);
    }

    public function get_debuginfo ()
    {
        return $this->debuginfo;
    }

    public function set_option ($option, $parm, $force = false)
    {
        $debug = $this->get_option('debug');
        if ($debug == true && $option != "debug")
        {
            $debug = $this->get_option('debug');
            $value = plg_rgraph_valid_options::options($option);
            if (! is_array($value) && $force == false)
                $this->set_debuginfo("option not set: " . $option);
        }

        if ($force === true)
        {
            $this->$option = $parm;
            return true;
        }

        $value = plg_rgraph_valid_options::options($option);
        if ($value != - 1)
        {
            $type = gettype($parm);

            if ($type == $value['type'])
            {
                if ($value['type'] == "array")
                {
                    if (! empty($value['types']))
                    {
                        if ($value['types'] == "integer" || $value['types'] == "double")
                            $parm = $this->set_numeric($parm);
                        elseif ($value['types'] == "boolean")
                            $parm = $this->set_boolean($parm);
                    }
                }

                if ($option == "show_labels")
                    $this->labels = $parm;

                if ($option == "tooltipsCss")
                    $parm = 'JAVA_VAR{ ' . implode(",", $parm) . ' }JAVA_VAR';

                if ($option == "contextmenu")
                {
                    if (! is_array($parm[0]))
                    {
                        if (isset($parm[1]))
                            $parm[1] = "JAVA_VAR" . $parm[1] . "JAVA_VAR";
                    }
                    else
                    {
                        for ($i = 0; $i < count($parm); $i ++)
                        {
                            if (isset($parm[$i][1]))
                                $parm[$i][1] = "JAVA_VAR" . $parm[$i][1] . "JAVA_VAR";
                        }
                    }
                }
                if ($option == "eventsClick" || $option == "eventsMousemove")
                    $parm = 'JAVA_VAR' . $parm . 'JAVA_VAR';

                if ($option == "fill.click" || $option == "fill.mousemove")
                    $parm = 'JAVA_VAR' . $parm . 'JAVA_VAR';

                if ($option == "tooltipsOverride" || $option == "xscaleFormatter")
                    $parm = 'JAVA_VAR' . $parm . 'JAVA_VAR';

                $this->$option = $parm;

                return true;
            }
            elseif ($type == "integer" && $value['type'] == "double")
            {
                $this->$option = $parm;
                return true;
            }
            elseif ($value['type'] == "double" && is_numeric($parm))
            {
                settype($parm, "float");
                $this->$option = $parm;
                return true;
            }
            elseif ($value['type'] == "integer" && is_numeric($parm))
            {
                settype($parm, "integer");
                $this->$option = $parm;
                return true;
            }
            elseif ($value['type'] == "boolean")
            {
                $this->$option = $this->set_boolean($parm);
                return true;
            }

            elseif ($value['type'] == "array")
            {
                return false;
            }

            elseif ($parm === null)
            {
                $this->$option = null;
            }
            else
                return false;
        }

        return - 1;
    }

    public function get_optionType ($option)
    {
        $value = plg_rgraph_valid_options::options($option);
        if ($value != - 1)
        {
            return $value['type'];
        }
        return - 1;
    }

    function responsiveOptions ($scale)
    {
        $checkOptions = array(
                "textSize",
                "titleSize",
                "titleSubtitleSize",
                "titleTopSize",
                "titleTopSubtitleSize",
                "titleBottomSize",
                "titleLeftSize",
                "titleRightSize",
                "xaxisTitleSize",
                "yaxisTitleSize",
                "xaxisLabelsSize",
                "yaxisLabelsSize",
                "labelsValueSize",
                "labelsValueTextSize",
                "labelsInbarSize",
                "labelsIngraphSize",
                "labelsAboveSize",
                "labelsAngledSize",
                "labelsAngledDownSize",
                "labelsAngledUpSize",
                "labelsAngledLevelSize",
                "marginTop",
                "marginBottom",
                "marginRight",
                "marginLeft",
                "radius",
                "bubbleWidth",
                "tickmarksSize"
        );
        for ($i = 0; $i < count($checkOptions); $i ++)
        {
            $value = $this->get_option($checkOptions[$i]);
            if ($value)
                $props[$checkOptions[$i]] = intval($value * $scale);
        }

        return json_encode($props);
    }

    public function getResponsiveFunction ($parms)
    {
        if (! is_array($parms))
            return "";

        $scale = floatval($parms[0]);

        if ($scale > 0 && count($parms) == 5)
        {

            $min_width = intval($parms[1]);
            $max_width = intval($parms[2]);
            $width = intval($parms[3]);
            $height = intval($parms[4]);

            // check input
            if ($scale < 0.1 || $scale > 0.5)
                $scale = 0.25;
            if ($min_width > $max_width)
            {
                $min_width = 800;
                $max_width = 1280;
            }

            $fs = 1 - $scale;
            $fl = 1 + $scale;

            $small = "{maxWidth:" . $min_width . ", width:" . floor($width * $fs / 10) * 10 . ",height:" . floor($height * $fs / 10) * 10 . ",";
            $medium = "{maxWidth:" . $max_width . ", width:" . $width . ",height:" . $height . ",";
            $large = "{maxWidth: null, width:" . floor($width * $fl / 10) * 10 . ",height:" . floor($height * $fl / 10) * 10 . ",";

            $small .= "options:" . $this->responsiveOptions($fs) . "},";
            $medium .= "options:" . $this->responsiveOptions(1) . "},";
            $large .= "options:" . $this->responsiveOptions($fl) . "}";

            $function = "\n.responsive([" . $small . " " . $medium . " " . $large . " ])";
            return $function;
        }
        else
            return "";
    }

    function numberFormat ($number_format, $type)
    {
        if ($number_format != '')
        {
            $noaxis_charts = array(
                    "check",
                    "Pie",
                    "Fuel",
                    "Funnel",
                    "Gauge",
                    "Meter",
                    "Odometer",
                    "Radar",
                    "Rose",
                    "RScatter"
            );

            switch ($number_format)
            {
                case "a":
                    $point = ".";
                    $sep = "\0";
                    break;

                case "b":
                    $point = ",";
                    $sep = "\0";
                    break;

                case "c":
                    $point = ",";
                    $sep = ".";
                    break;

                case "d":
                    $point = ".";
                    $sep = ",";
                    break;
            }

            $this->set_option("scalePoint", $point, true);
            $this->set_option("scaleThousand", $sep, true);

            if ($type == 'Gauge' || $type == 'Meter')
            {
                $this->set_option("labelsValuePoint", $point, true);
                $this->set_option("labelsValueThousand", $sep, true);
            }
            else
            {
                if (array_search($type, $noaxis_charts) == false)
                {
                    $this->set_option("xaxisScalePoint", $point, true);
                    $this->set_option("xaxisScaleThousand", $sep, true);
                    $this->set_option("yaxisScalePoint", $point, true);
                    $this->set_option("yaxisScaleThousand", $sep, true);
                }

                if ($type == "Pie")
                {
                    $this->set_option("labelsIngraphDecimals", $this->scaleDecimals);
                    $this->set_option("labelsIngraphPoint", $point);
                    $this->set_option("labelsIngraphThousand", $sep);
                }
            }
        }
    }

    public function set_tooltip ($data, $string = "#val#", $type = "Bar")
    {
        if (! isset($this->tooltipsFormattedThousand))
            $this->tooltipsFormattedThousand = $this->scaleThousand;
        if (! isset($this->tooltipsFormattedDecimals))
            $this->tooltipsFormattedDecimals = $this->scaleDecimals;
        if (! isset($this->tooltipsFormattedPoint))
            $this->tooltipsFormattedPoint = $this->scalePoint;
        $num_dec = $this->tooltipsFormattedDecimals;
        $sep_thousand = $this->tooltipsFormattedThousand;
        $sep_dec = $this->tooltipsFormattedPoint;

        if (isset($this->origdata) && $this->log_scale = "1")
        {
            $string = str_replace("#val#", "%{property:origdata[%{index}]}", $string);
        }

        if ($type != "Gantt")
            $string = str_replace("#val#", "%{value_formatted}", $string);

        if (! is_array($data[0]))
        {
            $total = array_sum($data); // $num_dec, $sep_dec, $sep_thousand);
            $string = str_replace("#total#", $total, $string);

            for ($i = 0; $i < count($data); $i ++)
            {
                if ($total != 0)
                    $percent[$i] = number_format(100 * $data[$i] / $total, $num_dec);
            }
        }
        elseif (is_array($data[0]) && $type != "Gantt")
        {
            // calculate totals / group
            for ($i = 0; $i < count($data); $i ++)
            {
                $total[$i] = 0;
                for ($j = 0; $j < count($data[$i]); $j ++)
                    $total[$i] += $data[$i][$j]; // $sep_dec, $sep_thousand);
            }
            for ($i = 0; $i < count($total); $i ++)
                $total[$i] = number_format($total[$i], $num_dec, $sep_dec, $sep_thousand);
        }

        switch ($type)
        {
            case "Gantt":

                for ($i = 0; $i < count($data); $i ++)
                {
                    $start[$i] = number_format($data[$i]['start'], 0);
                    $complete[$i] = number_format($data[$i]['complete'], 0);
                    $duration[$i] = number_format($data[$i]['duration'], 0);
                }

                $this->start = $start;
                $this->duration = $duration;
                $this->complete = $complete;

                $string = str_replace("#val0#", "%{property:start[%{dataset}]}", $string);
                $string = str_replace("#val1#", "%{property:duration[%{dataset}]}", $string);
                $string = str_replace("#val2#", "%{property:complete[%{dataset}]}", $string);
                $string = str_replace("#label#", "%{property:yaxisLabels[%{dataset}]}", $string);
                break;

            case "Bipolar":
                if (! empty($this->key))
                    $string = str_replace("#key#", "%{property:key[%{group}]}", $string);
                $string = str_replace("#label#", "%{property:labels[%{dataset2}]}", $string);
                break;

            case "Pie":
            case "Funnel":
            case "Waterfall":
            case "Line":
            case "Radar":
                if (! empty($this->key))
                    $string = str_replace("#key#", "%{property:key[%{group}]}", $string);
                if (strpos($string, "#percent#") !== false)
                {
                    $this->percent = $percent;
                    $string = str_replace("#percent#", "%{property:percent[%{index}]}", $string);
                }
                if (strpos($string, "#total#") !== false)
                {
                    $this->total = $total;
                    $string = str_replace("#total#", "%{property:total[%{group}]}", $string);
                }

                if (isset($this->origlabels))
                    $string = str_replace("#label#", "%{property:origlabels[%{index}]}", $string);

                if (isset($this->labels) && ! isset($this->origlabels))
                    $string = str_replace("#label#", "%{property:labels[%{index}]}", $string);
                break;

            default:
                if (! empty($this->key))
                    $string = str_replace("#key#", "%{property:key[%{index}]}", $string);
                if (strpos($string, "#total#") !== false)
                {
                    $this->total = $total;
                    $string = str_replace("#total#", "%{property:total[%{group}]}", $string);
                }
                if (isset($this->origlabels))
                    $string = str_replace("#label#", "%{property:origlabels[%{group}]}", $string);

                if (isset($this->labels) && ! isset($this->origlabels))
                    $string = str_replace("#label#", "%{property:labels[%{group}]}", $string);
                break;
        }

        $this->tooltips = $string;
        return true;
    }

    public static function xlabelsteps ($labels, $num)
    {
        if (! is_array($labels))
            return false;

        $new_labels = array();

        for ($i = 0; $i < count($labels); $i ++)
        {
            if ($i % $num == 0)
                $new_labels[$i] = $labels[$i];
            else
                $new_labels[$i] = "";
        }
        return $new_labels;
    }

    public function get_colors ()
    {
        if (isset($this->colors))
            return $this->colors;
    }

    public function set_gradientColors ()
    {
        for ($i = 0; $i < count($this->colors); $i ++)
        {
            if (strpos($this->colors[$i], 'Gradient') === false)
                $this->colors[$i] = $this->set_gradientColor($this->colors[$i]);
                //echo  "<br>" . $this->colors[$i];
        }
        
    }
      
    public function set_gradientColor ($color)
    {
        $gradient = "Gradient(" . $this->setOpacity($color, 0.85) . ":";
        //$gradient .= $this->setOpacity($color, 0.8) . ":";
        $gradient .= $this->setOpacity($color, 0.6) . ")";
            
        return $gradient;
    }

    public function setColorOpacity ($opacity = 0.8)
    {
        if (! is_array($this->colors))
            return false;

        for ($i = 0; $i < count($this->colors); $i ++)
            $this->colors[$i] = $this->setOpacity($this->colors[$i], $opacity);

        return true;
    }

    public static function setOpacity ($color, $opacity = 1.0)
    {
        if (empty($color))
            return false;

        if ($opacity > 1)
            $opacity = 1.0;
        
        if ( preg_match('/^[0-9a-f]+$/i', $color) )
           $color = "#" . $color;
        
        if ($color[0] == '#')
        {
            $hex = dechex(intval(255 * $opacity));
            if (strlen($hex) < 2)
                $hex = "0" . $hex;

            elseif (strlen($color) == 7)
                $color = $color . $hex;
            
             elseif (strlen($color) == 9)
                $color = substr($color,0,7) . $hex;
            
            elseif (strlen($color) == 4)
            {
                $tmp = "#";
                for ($i = 1; $i < 4; $i ++)
                {
                    $tmp = $tmp . $color[$i] . $color[$i];
                }
                $color = $tmp . $hex;
            }

            return $color;
        }

        if (substr($color, 0, 4) == 'rgb(')
        {
            $tmp = explode(",", $color);
            $tmp[0] = str_replace("rgb", "rgba", $tmp[0]);
            $tmp[2] = str_replace(")", "", $tmp[2]);
            array_push($tmp, $opacity);
            $color = implode(",", $tmp) . ")";
            return ($color);
        }

        if (substr($color, 0, 4) == 'rgba')
        {
            $tmp = explode(",", $color);
            $tmp[3] = $opacity . ")";
            $color = implode(",", $tmp);
            return ($color);
        }

        return $color;
    }

    public function set_optionsviaArray ($parms, $nocheck = false)
    {
        if (! is_array($parms))
            return - 1;

        $result = true;

        foreach ($parms as $key => $value)
        {
            $type = $this->get_optionType($key);

            if ($type != - 1)
            {
                if ($type == "array")
                {
                    if ($key == "tooltips" && strpos($value, "function") !== false)
                    {
                        $trans = array(
                                "[" => "{",
                                "]" => "}"
                        );
                        $value = strtr($value, $trans);
                        $nocheck = true;
                    }

                    elseif (strpos($value, "rgb") !== false && $key == "colors")
                    {
                        $value = $this->buildcolorsArray($value);
                    }

                    else
                    {
                        if (strpos($value, "|"))
                            $tmp = explode("|", $value);
                        else if (strpos($value, "rgb") !== false)
                            $value = $this->buildcolorsArray($value);
                        else
                            $value = explode(",", $value);

                        if (isset($tmp))
                        {
                            $value = array();
                            for ($i = 0; $i < count($tmp); $i ++)
                            {
                                if (strpos($tmp[$i], "rgb") !== false)
                                    $value[$i] = $this->buildcolorsArray($tmp[$i]);
                                else
                                {
                                    $value[$i] = explode(",", $tmp[$i]);
                                    $value[$i] = $this->set_numeric($value[$i]);
                                }
                            }
                        }
                        // $value = $this->set_numeric ($value);
                    }
                }

                $rc = true;
                if ($value !== null && $value != "")
                {
                    if ($value === "null")
                        $value = null;
                    $rc = $this->set_option($key, $value, $nocheck);
                }
                if ($rc !== true)
                {
                    if ($this->debug)
                        array_push($this->debuginfo, "option not set: " . $key);
                    $result = 2;
                }
            }
            else
            {
                if ($this->debug)
                    array_push($this->debuginfo, "option not set: " . $key);
                $result = 2;
            }
        }
        return $result;
    }

    public function buildcolorsArray ($color_values)
    {
        $tmp = explode(",", $color_values);
        $tmp = $this->set_numeric($tmp);
        $k = 0;
        $new_values = array();
        for ($i = 0; $i < count($tmp);)
        {
            if (strpos($tmp[$i], "rgb") !== false)
            {
                $count = 3;

                if (strpos($tmp[$i], "rgba") !== false)
                    $count = 4;

                $rgb_array = array();
                for ($j = 0; $j < $count; $j ++)
                {
                    array_push($rgb_array, $tmp[$i]);
                    $i ++;
                }
                $new_values[$k] = implode(",", $rgb_array);
            }
            else
            {
                $new_values[$k] = $tmp[$i];
                $i ++;
            }
            $k ++;
        }

        return $new_values;
    }

    public static function set_boolean ($parm)
    {
        if (! is_array($parm))
        {
            if (strtolower($parm) == 'true' || $parm == "1")
                return true;
            elseif (strtolower($parm) == 'false' || $parm == '0')
                return false;
            else
                return settype($parm, 'boolean');
        }
        else
        {
            for ($i = 0; $i < count($parm); $i ++)
            {
                if (strtolower($parm[$i]) == 'true' || $parm[$i] == '1')
                    $parm[$i] = true;
                elseif (strtolower($parm[$i]) == 'false' || $parm[$i] == '0')
                    $parm[$i] = false;
                else
                    $parm[$i] = settype($parm, 'boolean');
            }

            return $parm;
        }

        return $parm;
    }

    public static function set_numeric ($data)
    {
        if (! is_array($data[0]))
        {
            for ($i = 0; $i < count($data); $i ++)
            {
                $data[$i] = str_replace(",", ".", $data[$i]);
                if ($data[$i] == "")
                    $data[$i] = null;
                if (is_numeric($data[$i]))
                    settype($data[$i], "float");
            }
        }
        else
        {
            for ($i = 0; $i < count($data); $i ++)
                for ($j = 0; $j < count($data[$i]); $j ++)
                {
                    $data[$i][$j] = str_replace(",", ".", $data[$i][$j]);
                    if ($data[$i][$j] == "")
                        $data[$i][$j] = null;
                    if (is_numeric($data[$i][$j]))
                        settype($data[$i][$j], "float");
                }
        }
        return $data;
    }

    public function truncateLabels ($labels, $size, $chars = null)
    {
        if ($size == '0')
            return $labels;

        if (! is_array($labels))
            return false;

        $this->set_option('origlabels', $labels);

        for ($i = 0; $i < count($labels); $i ++)
        {
            if (strlen($labels[$i]) > $size)
                $labels[$i] = mb_substr($labels[$i], 0, $size) . $chars;
        }

        return $labels;
    }

    public static function getRandomColor ()
    {
        $hex_rgb = dechex(rand(0, 255)) . dechex(rand(0, 255)) . dechex(rand(0, 255));
        if (strlen($hex_rgb) == 4)
        {
            $hex_rgb = $hex_rgb . rand(10, 99);
        }
        elseif (strlen($hex_rgb) == 5)
        {
            $hex_rgb = $hex_rgb . rand(0, 9);
        }
        $color = "#" . $hex_rgb;

        return $color;
    }

    public static function getRandomColors ($count = 1)
    {
        $colors = array();

        for ($i = 0; $i < $count; $i ++)
            $colors[$i] = getRandomColor();

        return $colors;
    }
}

?>